package cn.com.infosec.crypto.macs;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.Mac;
import cn.com.infosec.crypto.params.KeyParameter;
import cn.com.infosec.crypto.params.ParametersWithIV;

public class VMPCMac implements Mac {
   private byte g;
   private byte n = 0;
   private byte[] P = null;
   private byte s = 0;
   private byte[] T;
   private byte[] workingIV;
   private byte[] workingKey;
   private byte x1;
   private byte x2;
   private byte x3;
   private byte x4;

   private void initKey(byte[] keyBytes, byte[] ivBytes) {
      this.s = 0;
      this.P = new byte[256];

      int var3;
      for(var3 = 0; var3 < 256; ++var3) {
         this.P[var3] = (byte)var3;
      }

      int var8;
      byte[] var10000;
      byte var10001;
      for(var3 = 0; var3 < 768; ++var3) {
         byte[] var4;
         var10000 = var4 = this.P;
         int var5;
         byte var6;
         var10001 = var6 = var10000[this.s + var4[var5 = var3 & 255] + keyBytes[var3 % keyBytes.length] & 255];
         this.s = var6;
         var8 = var4[var5];
         var4[var5] = var4[var6 & 255];
         var10000[var10001 & 255] = (byte)var8;
      }

      for(int keyBytes1 = 0; keyBytes1 < 768; ++keyBytes1) {
         byte[] var9;
         var10000 = var9 = this.P;
         byte var11;
         var10001 = var11 = var10000[this.s + var9[var8 = keyBytes1 & 255] + ivBytes[keyBytes1 % ivBytes.length] & 255];
         this.s = var11;
         byte var10 = var9[var8];
         var9[var8] = var9[var11 & 255];
         var10000[var10001 & 255] = var10;
      }

      this.n = 0;
   }

   public int doFinal(byte[] out, int outOff) {
      int var3;
      byte[] var4;
      int var5;
      byte var6;
      int var7;
      byte var8;
      byte var11;
      byte var10001;
      for(var3 = 1; var3 < 25; ++var3) {
         var4 = this.P;
         var5 = this.s;
         var10001 = var6 = this.n;
         byte var10003 = (byte)(var5 = var4[var5 + var4[var6 & 255] & 255]);
         byte var10006 = (byte)var5;
         byte var10007 = var6;
         byte var10009 = var6;
         this.s = (byte)var5;
         var6 = this.x4;
         byte var10018 = (byte)(var7 = this.x3);
         this.x4 = var6 = var4[var6 + var7 + var3 & 255];
         this.x3 = var8 = var4[var10018 + (var7 = this.x2) + var3 & 255];
         byte var9;
         byte var10015 = var9 = this.x1;
         int var10;
         this.x2 = (byte)(var10 = var4[var7 + var9 + var3 & 255]);
         this.x1 = (byte)(var5 = var4[var10015 + var5 + var3 & 255]);
         byte[] var14 = this.T;
         byte var10011 = var9 = this.g;
         var7 = var9 & 31;
         var14[var7] = (byte)(var14[var9 & 31] ^ var5);
         var5 = var9 + 1 & 31;
         var14[var5] = (byte)(var14[var9 + 1 & 31] ^ var10);
         var10 = var9 + 2 & 31;
         var14[var10] = (byte)(var14[var9 + 2 & 31] ^ var8);
         var10 = var9 + 3 & 31;
         var14[var10] = (byte)(var14[var9 + 3 & 31] ^ var6);
         this.g = (byte)(var10011 + 4 & 31);
         var11 = var4[var10009 & 255];
         var5 = var10007 & 255;
         var4[var5] = var4[var10006 & 255];
         var4[var10003 & 255] = var11;
         this.n = (byte)(var10001 + 1 & 255);
      }

      byte[] var10000;
      for(var3 = 0; var3 < 768; ++var3) {
         var10000 = var4 = this.P;
         var10001 = var6 = var10000[this.s + var4[var5 = var3 & 255] + this.T[var3 & 31] & 255];
         this.s = var6;
         var11 = var4[var5];
         var4[var5] = var4[var6 & 255];
         var10000[var10001 & 255] = var11;
      }

      var11 = 20;
      byte[] var12 = new byte[20];

      for(var3 = 0; var3 < 20; ++var3) {
         byte[] var13;
         var10000 = var13 = this.P;
         var10001 = var8 = var10000[this.s + var13[var7 = var3 & 255] & 255];
         this.s = var8;
         var12[var3] = var13[var13[var13[var8 & 255] & 255] + 1 & 255];
         var6 = var13[var7];
         var13[var7] = var13[var8 & 255];
         var10000[var10001 & 255] = var6;
      }

      System.arraycopy(var12, 0, out, outOff, var11);
      this.reset();
      return var11;
   }

   public String getAlgorithmName() {
      return "VMPC-MAC";
   }

   public int getMacSize() {
      return 20;
   }

   public void init(CipherParameters params) {
      if (params instanceof ParametersWithIV) {
         ParametersWithIV var10000 = (ParametersWithIV)params;
         ParametersWithIV params1;
         KeyParameter var2 = (KeyParameter)(params1 = (ParametersWithIV)params).getParameters();
         if (var10000.getParameters() instanceof KeyParameter) {
            byte[] params2;
            byte[] var3 = params2 = params1.getIV();
            this.workingIV = params2;
            if (var3 != null && params2.length >= 1 && params2.length <= 768) {
               this.workingKey = var2.getKey();
               this.reset();
            } else {
               throw new IllegalArgumentException("VMPC-MAC requires 1 to 768 bytes of IV");
            }
         } else {
            throw new IllegalArgumentException("VMPC-MAC Init parameters must include a key");
         }
      } else {
         throw new IllegalArgumentException("VMPC-MAC Init parameters must include an IV");
      }
   }

   public void reset() {
      byte[] var1 = this.workingKey;
      this.initKey(var1, this.workingIV);
      this.n = 0;
      this.x4 = 0;
      this.x3 = 0;
      this.x2 = 0;
      this.x1 = 0;
      this.g = 0;
      this.T = new byte[32];

      for(int var2 = 0; var2 < 32; ++var2) {
         this.T[var2] = 0;
      }

   }

   public void update(byte in) {
      VMPCMac var10000 = this;
      byte[] var2 = this.P;
      byte var3 = this.s;
      byte var4;
      byte var10001 = var4 = this.n;
      byte[] var10002 = var2;
      byte var10003 = var3 = var2[var3 + var2[var4 & 255] & 255];
      byte[] var10004 = var2;
      byte[] var10005 = var2;
      byte var10006 = var3;
      byte var10007 = var4;
      byte[] var10008 = var2;
      byte var10009 = var4;
      VMPCMac var10010 = this;
      VMPCMac var10011 = this;
      VMPCMac var10012 = this;
      VMPCMac var10013 = this;
      this.s = var3;
      in ^= var2[var2[var2[var3 & 255] & 255] + 1 & 255];
      var4 = this.x4;
      byte var5;
      byte var10018 = var5 = this.x3;
      this.x4 = var4 = var2[var4 + var5 & 255];
      byte var6;
      this.x3 = var6 = var2[var10018 + (var5 = this.x2) & 255];
      byte var7;
      byte var10015 = var7 = this.x1;
      int var8;
      this.x2 = (byte)(var8 = var2[var5 + var7 & 255]);
      var10013.x1 = (byte)(in = var2[var10015 + var3 + in & 255]);
      var2 = var10012.T;
      byte var11 = var3 = var10011.g;
      byte[] var12 = var2;
      byte[] var13 = var2;
      byte[] var10016 = var2;
      byte[] var10017 = var2;
      byte[] var10020 = var2;
      byte[] var10021 = var2;
      byte[] var10024 = var2;
      byte[] var10025 = var2;
      int var10 = var3 & 31;
      var10024[var10] = (byte)(var10025[var3 & 31] ^ in);
      in = var3 + 1 & 31;
      var10020[in] = (byte)(var10021[var3 + 1 & 31] ^ var8);
      var8 = var3 + 2 & 31;
      var10016[var8] = (byte)(var10017[var3 + 2 & 31] ^ var6);
      var8 = var3 + 3 & 31;
      var12[var8] = (byte)(var13[var3 + 3 & 31] ^ var4);
      var10010.g = (byte)(var11 + 4 & 31);
      byte var9 = var10008[var10009 & 255];
      in = var10007 & 255;
      var10004[in] = var10005[var10006 & 255];
      var10002[var10003 & 255] = var9;
      var10000.n = (byte)(var10001 + 1 & 255);
   }

   public void update(byte[] in, int inOff, int len) {
      if (inOff + len > in.length) {
         throw new DataLengthException("input buffer too short");
      } else {
         for(inOff = 0; inOff < len; ++inOff) {
            this.update(in[inOff]);
         }

      }
   }
}
